@@ -7,17 +7,27 @@ import warnings
 import h5py
 import numpy as np
 
+from distutils.version import LooseVersion
+
 from .compat import ChainMap, OrderedDict, unicode
 from .attrs import Attributes
 from .dimensions import Dimensions
 from .utils import Frozen
 
+try:
+    import h5pyd
+except ImportError:
+    no_h5pyd = True
+    h5_group_types = (h5py.Group,)
+else:
+    no_h5pyd = False
+    h5_group_types = (h5py.Group, h5pyd.Group)
 
-__version__ = '0.5.0'
+__version__ = '0.7.1'
 
 
-_NC_PROPERTIES = (u'version=1|h5netcdfversion=%s|hdf5libversion=%s'
-                  % (__version__, h5py.version.hdf5_version))
+_NC_PROPERTIES = (u'version=2,h5netcdf=%s,hdf5=%s,h5py=%s'
+                  % (__version__, h5py.version.hdf5_version, h5py.__version__))
 
 NOT_A_VARIABLE = b'This is a netCDF dimension but not a netCDF variable.'
 
@@ -197,6 +207,10 @@ class _LazyObjectLookup(Mapping):
             return self._objects[key]
 
 
+def _netcdf_dimension_but_not_variable(h5py_dataset):
+    return NOT_A_VARIABLE in h5py_dataset.attrs.get('NAME', b'')
+
+
 class Group(Mapping):
 
     _variable_cls = Variable
@@ -214,12 +228,15 @@ class Group(Mapping):
             self._dim_sizes = parent._dim_sizes.new_child()
             self._current_dim_sizes = parent._current_dim_sizes.new_child()
             self._dim_order = parent._dim_order.new_child()
+            self._all_h5groups = parent._all_h5groups.new_child(self._h5group)
 
         self._variables = _LazyObjectLookup(self, self._variable_cls)
         self._groups = _LazyObjectLookup(self, self._group_cls)
 
         for k, v in self._h5group.items():
-            if isinstance(v, h5py.Group):
+            if isinstance(v, h5_group_types):
+                # add to the groups collection if this is a h5py(d) Group
+                # instance
                 self._groups.add(k)
             else:
                 if v.attrs.get('CLASS') == b'DIMENSION_SCALE':
@@ -243,9 +260,9 @@ class Group(Mapping):
                         self._determine_current_dimension_size(k, current_size)
 
                     if dim_id is None:
-                        dim_id = len(self._dim_order)
+                        dim_id = self._root._get_next_dim_id()
                     self._dim_order[k] = dim_id
-                if NOT_A_VARIABLE not in v.attrs.get('NAME', b''):
+                if not _netcdf_dimension_but_not_variable(v):
                     var_name = k
                     if k.startswith('_nc4_non_coord_'):
                         var_name = k[len('_nc4_non_coord_'):]
@@ -269,15 +286,7 @@ class Group(Mapping):
         dim_variable = _find_dim(self._h5group, dim_name)
 
         if "REFERENCE_LIST" not in dim_variable.attrs:
-            if dim_variable.shape == (0,):
-                # NetCDF does not create the REFERENCE_LIST attribute if
-                # an unlimited dimension is of zero length. In this case it
-                # is safe to skip this dimension.
-                return max_size
-            else:  # pragma: no cover
-                raise ValueError(
-                    "Each dimension with an actual length must have a "
-                    "'REFERENCE_LIST' attribute.")
+            return max_size
 
         root = self._h5group["/"]
 
@@ -306,7 +315,7 @@ class Group(Mapping):
 
         self._dim_sizes[name] = size
         self._current_dim_sizes[name] = 0 if size is None else size
-        self._dim_order[name] = len(self._dim_order)
+        self._dim_order[name] = self._root._get_next_dim_id()
 
     @property
     def dimensions(self):
@@ -372,13 +381,6 @@ class Group(Mapping):
         else:
             self._root._check_valid_netcdf_dtype(dtype, stacklevel=stacklevel)
 
-        compression = kwargs.get('compression')
-        if compression not in {None, 'gzip'}:
-            _invalid_netcdf_feature('{} compression'.format(compression),
-                                    allow=self._root.invalid_netcdf,
-                                    file=self._root,
-                                    stacklevel=stacklevel)
-
         if 'scaleoffset' in kwargs:
             _invalid_netcdf_feature('scale-offset filters',
                                     allow=self._root.invalid_netcdf,
@@ -398,6 +400,14 @@ class Group(Mapping):
         if shape != maxshape:
             kwargs["maxshape"] = maxshape
 
+        # Clear dummy HDF5 datasets with this name that were created for a
+        # dimension scale without a corresponding variable.
+        if name in self.dimensions and name in self._h5group:
+            h5ds = self._h5group[name]
+            if _netcdf_dimension_but_not_variable(h5ds):
+                self._detach_dim_scale(name)
+                del self._h5group[name]
+
         self._h5group.create_dataset(h5name, shape, dtype=dtype,
                                      data=data, fillvalue=fillvalue,
                                      **kwargs)
@@ -447,6 +457,7 @@ class Group(Mapping):
         return len(self.variables) + len(self.groups)
 
     def _create_dim_scales(self):
+        """Create all necessary HDF5 dimension scale."""
         dim_order = self._dim_order.maps[0]
         for dim in sorted(dim_order, key=lambda d: dim_order[d]):
             if dim not in self._h5group:
@@ -471,19 +482,26 @@ class Group(Mapping):
         for subgroup in self.groups.values():
             subgroup._create_dim_scales()
 
-    def _attach_dim_scales(self, parent_h5groups=None):
-        if parent_h5groups is None:
-            all_h5groups = ChainMap(self._h5group)
-        else:
-            all_h5groups = parent_h5groups.new_child(self._h5group)
-
+    def _attach_dim_scales(self):
+        """Attach dimension scales to all variables."""
         for name, var in self.variables.items():
             if name not in self.dimensions:
                 for n, dim in enumerate(var.dimensions):
-                    var._h5ds.dims[n].attach_scale(all_h5groups[dim])
+                    var._h5ds.dims[n].attach_scale(self._all_h5groups[dim])
 
         for subgroup in self.groups.values():
-            subgroup._attach_dim_scales(all_h5groups)
+            subgroup._attach_dim_scales()
+
+    def _detach_dim_scale(self, name):
+        """Detach the dimension scale corresponding to a dimension name."""
+        for var in self.variables.values():
+            for n, dim in enumerate(var.dimensions):
+                if dim == name:
+                    var._h5ds.dims[n].detach_scale(self._all_h5groups[dim])
+
+        for subgroup in self.groups.values():
+            if dim not in subgroup._h5group:
+                subgroup._detach_dim_scale(name)
 
     @property
     def parent(self):
@@ -560,31 +578,64 @@ class Group(Mapping):
 class File(Group):
 
     def __init__(self, path, mode='a', invalid_netcdf=None, **kwargs):
-        self._preexisting_file = os.path.exists(path)
         try:
-            self._h5file = h5py.File(path, mode, **kwargs)
+            if isinstance(path, str):
+                if path.startswith(('http://', 'https://', 'hdf5://')):
+                    if no_h5pyd:
+                        raise ImportError(
+                            "No module named 'h5pyd'. h5pyd is required for "
+                            "opening urls: {}".format(path))
+                    try:
+                        with h5pyd.File(path, 'r') as f:  # noqa
+                            pass
+                        self._preexisting_file = True
+                    except IOError:
+                        self._preexisting_file = False
+                    self._h5file = h5pyd.File(path, mode, **kwargs)
+                else:
+                    self._preexisting_file = os.path.exists(path)
+                    self._h5file = h5py.File(path, mode, **kwargs)
+            else:  # file-like object
+                if h5py.__version__ < LooseVersion('2.9.0'):
+                    raise TypeError(
+                        "h5py version ({}) must be greater than 2.9.0 to load "
+                        "file-like objects.".format(h5py.__version__))
+                else:
+                    self._preexisting_file = mode in {'r', 'r+', 'a'}
+                    self._h5file = h5py.File(path, mode, **kwargs)
         except Exception:
             self._closed = True
             raise
         else:
             self._closed = False
 
-        # Three maps to keep track of dimensions in terms of size (might be
-        # unlimited), current size (identical to size for limited dimensions),
-        # and their position.
-        self._dim_sizes = ChainMap()
-        self._current_dim_sizes = ChainMap()
-        self._dim_order = ChainMap()
-
         self._mode = mode
         self._root = self
         self._h5path = '/'
         self.invalid_netcdf = invalid_netcdf
         # If invalid_netcdf is None, we'll disable writing _NCProperties only
         # if we actually use invalid NetCDF features.
-        self._write_ncproperties = (invalid_netcdf is not True)
+        self._write_ncproperties = invalid_netcdf is not True
+
+        # These maps keep track of dimensions in terms of size (might be
+        # unlimited), current size (identical to size for limited dimensions),
+        # their position, and look-up for HDF5 datasets corresponding to a
+        # dimension.
+        self._dim_sizes = ChainMap()
+        self._current_dim_sizes = ChainMap()
+        self._dim_order = ChainMap()
+        self._all_h5groups = ChainMap(self._h5group)
+
+        # used for picking numbers to use in self._dim_order
+        self._next_dim_id = 0
+
         super(File, self).__init__(self, self._h5path)
 
+    def _get_next_dim_id(self):
+        dim_id = self._next_dim_id
+        self._next_dim_id += 1
+        return dim_id
+
     def _check_valid_netcdf_dtype(self, dtype, stacklevel=3):
         dtype = np.dtype(dtype)
 
@@ -605,7 +656,7 @@ class File(Group):
             _invalid_netcdf_feature('{} dtypes'.format(description),
                                     allow=self.invalid_netcdf,
                                     file=self,
-                                    stacklevel=stacklevel+1)
+                                    stacklevel=stacklevel + 1)
 
     @property
     def mode(self):
